# -*- coding: UTF-8 -*-

import os
from common import filesmodify, logger
import datetime
import time
import codecs
import xml.dom.minidom
import re
import xml.etree.ElementTree as ET

logInfo = "CRYSTAL converter: "

saleFiles = set()

countPchs = {}          # количество чеков продаж
countRetPchs = {}            # количество чеков возврата

cashPchs = {}
cashlessPchs = {}

cashRetPchs = {}
cashlessRetPchs = {}

cashAmount = {}         # сумма продаж за наличные
cashlessAmount = {}     # сумма продаж за б/наличные
creditAmount = {}       # сумма продаж в кредит

cashRetAmount = {}      # сумма возвратов наличных
cashlessRetAmount = {}  # сумма возвратов б/наличных
creditRetAmount = {}    # сумма возвратов в кредит

amountDisc = {}
amountRetDisc = {}

countIncome = {}
countWithdrawal = {}

cashIn = {}
cashOut = {}

glUserNames = {}


def addZReport(zreports, shift, node):
    shiftNum = str(shift['shift'])

    zreport = ET.SubElement(zreports, 'zreport')

    reportType = ET.SubElement(zreport, 'reportType')
    reportType.text = 'ZReport'
    shiftNumber = ET.SubElement(zreport, 'shiftNumber')
    shiftNumber.text = shiftNum
    shopNumber = ET.SubElement(zreport, 'shopNumber')
    shopNumber.text = str(shift['shopcode'])
    docNumber = ET.SubElement(zreport, 'docNumber')
    docNumber.text = str('')
    cashNumber = ET.SubElement(zreport, 'cashNumber')
    cashNumber.text = str(shift['cashCode'])
    serialCashNumber = ET.SubElement(zreport, 'serialCashNumber')
    serialCashNumber.text = str('')
    userTabNumber = ET.SubElement(zreport, 'userTabNumber')
    userTabNumber.text = str(shift['userCode'])
    userName = ET.SubElement(zreport, 'userName')

    if shift['shift'] in glUserNames:
        if shift['userCode'] in glUserNames[shift['shift']]:
            userName.text = glUserNames[shift['shift']][shift['userCode']]
        else:
            userName.text = "SHIFT_HASNT_CASHIER_WITH_CODE : %s" %shift['userCode']
    else:
        userName.text = "BAD_SHIFT : %s" % shift['shift']

    kkms = {}
    if shift and shift['kkms']:
        kkms = shift['kkms'][0]

    dateOperDay = ET.SubElement(zreport, 'dateOperDay')
    dateOperDay.text = formatDate(shift['timeBeg'])
    dateShiftClose = ET.SubElement(zreport, 'dateShiftClose')
    dateShiftClose.text = formatDate(shift['timeEnd'])
    dateShiftOpen = ET.SubElement(zreport, 'dateShiftOpen')
    dateShiftOpen.text = formatDate(shift['timeBeg'])
    countCashPurchase = ET.SubElement(zreport, 'countCashPurchase')
    countCashPurchase.text = str(cashPchs[shiftNum])
    countCashlessPurchase = ET.SubElement(zreport, 'countCashlessPurchase')
    countCashlessPurchase.text = str(cashlessPchs[shiftNum])
    countPurchase = ET.SubElement(zreport, 'countPurchase')
    countPurchase.text = str(countPchs[shiftNum])
    countCashReturn = ET.SubElement(zreport, 'countCashReturn')
    countCashReturn.text = str(cashRetPchs[shiftNum])
    countCashlessReturn= ET.SubElement(zreport, 'countCashlessReturn')
    countCashlessReturn.text = str(cashlessRetPchs[shiftNum])
    countReturn= ET.SubElement(zreport, 'countReturn')
    countReturn.text = str(countRetPchs[shiftNum])
    counterIncoming = ET.SubElement(zreport, 'counterIncoming')
    counterIncoming.text = str(countIncome[shiftNum])
    counterWithdrawal = ET.SubElement(zreport, 'counterWithdrawal')
    counterWithdrawal.text = str(countWithdrawal[shiftNum])
    amountByCashPurchase= ET.SubElement(zreport, 'amountByCashPurchase')            # сумма продаж за наличные
    amountByCashPurchase.text = str(cashAmount[shiftNum])
    amountByCashlessPurchase= ET.SubElement(zreport, 'amountByCashlessPurchase')    # сумма продаж за б/наличные
    amountByCashlessPurchase.text = str(cashlessAmount[shiftNum])
    amountByPurchaseFiscal = ET.SubElement(zreport, 'amountByPurchaseFiscal')       # сумма продаж из ФП
    amountByPurchaseFiscal.text = str(kkms.get('sumSale', 0.0))
    amountByCashReturn = ET.SubElement(zreport, 'amountByCashReturn')
    amountByCashReturn.text = str(cashRetAmount[shiftNum])
    amountByCashlessReturn = ET.SubElement(zreport, 'amountByCashlessReturn')
    amountByCashlessReturn.text = str(cashlessRetAmount[shiftNum])
    amountByReturnFiscal = ET.SubElement(zreport, 'amountByReturnFiscal')           # сумма возвратов из ФП
    amountByReturnFiscal.text = str(kkms.get('sumBack', 0.0))
    amountCashIn = ET.SubElement(zreport, 'amountCashIn')
    amountCashIn.text = str(cashIn[shiftNum])
    amountCashOut = ET.SubElement(zreport, 'amountCashOut')
    amountCashOut.text = str(cashOut[shiftNum])
    amountCashDiscount = ET.SubElement(zreport, 'amountCashDiscount')
    amountCashDiscount.text = str(amountDisc[shiftNum])
    returnDiscountCashPay = ET.SubElement(zreport, 'returnDiscountCashPay')
    returnDiscountCashPay.text = str(amountRetDisc[shiftNum])
    incresentTotalStart= ET.SubElement(zreport, 'incresentTotalStart')
    incresentTotalStart.text = '0'
    incresentTotalFinish= ET.SubElement(zreport, 'incresentTotalFinish')
    incresentTotalFinish.text = str(int(kkms.get('sumSale', 0) * 100))
    incresentTotalReturnStart = ET.SubElement(zreport, 'incresentTotalReturnStart')
    incresentTotalReturnStart.text = '0'
    incresentTotalReturnFinish = ET.SubElement(zreport, 'incresentTotalReturnFinish')
    incresentTotalReturnFinish.text = str(int(kkms.get('sumBack', 0) * 100))
    factoryCashNumber = ET.SubElement(zreport, 'factoryCashNumber')
    factoryCashNumber.text = str(int(kkms.get('serialNum', 0)))
    cashName = ET.SubElement(zreport, 'cashName')
    cashName.text = kkms.get('modelName', '')
    inn = ET.SubElement(zreport, 'inn')
    inn.text = ''

    payments = ET.SubElement(zreport, 'payments')

    paymentCash = ET.SubElement(payments, 'payment')
    paymentCash.set('amountPurchase', str(cashAmount[shiftNum]))
    paymentCash.set('amountReturn', str(cashRetAmount[shiftNum]))
    paymentCash.set('typeClass', 'CashPaymentEntity')
    paymentCashless = ET.SubElement(payments, 'payment')

    paymentCashless.set('amountPurchase', str(cashlessAmount[shiftNum]))
    paymentCashless.set('amountReturn', str(cashlessRetAmount[shiftNum]))
    paymentCashless.set('typeClass', 'BankCardPaymentEntity')
    paymentCredit = ET.SubElement(payments, 'payment')

    paymentCredit.set('amountPurchase', str(creditAmount[shiftNum]))
    paymentCredit.set('amountReturn', str(creditRetAmount[shiftNum]))
    paymentCredit.set('typeClass', 'ConsumerCreditPaymentEntity')


def addPaymentBankProperties(payment):
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'terminal.number')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'cash.transaction.date')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'card.hash')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'card.type')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'cash.transaction.id')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'card.number')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'auth.code')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'bank.type')
    pl.set('value', '')


def addPaymentCreditProperties(payment):
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'contract.bank')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'contract.bank.product')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'consumer.name')
    pl.set('value', '')
    pl = ET.SubElement(payment, 'plugin-property')
    pl.set('key', 'contract.number')
    pl.set('value', '')


def zrepFieldsFormDiscount(amount, check):
    global amountDisc, amountRetDisc

    if str(check['docType']) == '1':
        amountDisc[str(check['shift'])] = round(amountDisc[str(check['shift'])] + float(amount), 2)
    else:
        amountRetDisc[str(check['shift'])] += float(amount)


def zrepFieldsForm(paymentsMap, check):
    global cashPchs, cashlessPchs, cashRetPchs, cashlessRetPchs, cashAmount, cashlessAmount, cashRetAmount, cashlessRetAmount, creditAmount, creditRetAmount

    docType = str(check['docType'])
    shiftNum = str(check['shift'])
    if docType == '1':
        countPchs[str(check['shift'])] += 1
    elif docType in ['2', '25']:
        countRetPchs[str(check['shift'])] += 1

    for paytype in paymentsMap:
        amount = paymentsMap[paytype]

        if paytype == "cash":
            if docType == '1':
                cashPchs[shiftNum] += 1
                cashAmount[shiftNum] = round(cashAmount[shiftNum] + float(amount), 2)
            else:
                cashRetPchs[shiftNum] += 1
                cashRetAmount[shiftNum] += float(amount)
        elif paytype == "bank":
            if docType == '1':
                cashlessPchs[shiftNum] += 1
                cashlessAmount[shiftNum] += float(amount)
            else:
                cashlessRetPchs[shiftNum] += 1
                cashlessRetAmount[shiftNum] += float(amount)
        else:
            if docType == '1':
                # cashlessPchs[shiftNum] += 1
                creditAmount[shiftNum] += float(amount)
            else:
                # cashlessRetPchs[shiftNum] += 1
                creditRetAmount[shiftNum] += float(amount)

def zresCashIncomeOutcome(check):
    global cashIn, cashOut, countIncome, countWithdrawal

    if str(check['docType']) == '3':
        countIncome[str(check['shift'])] += 1
        cashIn[str(check['shift'])] += check['docSum']
    else:
        countWithdrawal[str(check['shift'])] += 1
        cashOut[str(check['shift'])] += check['docSum']

def zrepFillShifts(shift):
    countPchs[shift] = 0
    countRetPchs[shift] = 0

    cashPchs[shift] = 0
    cashlessPchs[shift] = 0

    cashRetPchs[shift] = 0
    cashlessRetPchs[shift] = 0

    cashAmount[shift] = 0
    cashlessAmount[shift] = 0
    creditAmount[shift] = 0

    cashRetAmount[shift] = 0
    cashlessRetAmount[shift] = 0
    creditRetAmount[shift] = 0

    amountDisc[shift] = 0
    amountRetDisc[shift] = 0

    countIncome[shift] = 0
    countWithdrawal[shift] = 0

    cashIn[shift] = 0
    cashOut[shift] = 0


def getUTC():
    tz = -time.timezone // 3600
    utc = '+' if tz >= 0 else '-'
    tz = abs(tz)
    utc += ('0' if tz // 10 < 1 else '') + str(tz)
    utc += ":00"
    return utc


def formatDate(date):
    return date.strftime('%Y-%m-%dT%H:%M:%S.') + str(int(date.strftime('%f')) // 1000) + getUTC()


def addDiscounts(purchase, invent):
    amount = 0
    num = 0
    for disc in invent['discountPositions']:
        if disc['discSum'] > 0:
            amount += disc['discSum']
            num += 1
            discount = ET.SubElement(purchase, 'discount')

            discount.set('quantity', str(invent['quant']))
            discount.set('advertType', 'DISCOUNT')
            discount.set('AdvertActExternalCode', str(disc['discCode']))
            discount.set('AdvertActGUID', str(disc['discScheme']))

            if str(disc['isPositionDiscount']) == 'true':
                discount.set('isDiscountPurchase', 'false')
            else:
                discount.set('isDiscountPurchase', 'true')
            discount.set('amount', str(disc['discSum']))
            discount.set('goodCode', str(invent['inventCode']))
            discount.set('positionId', str(invent['posNum']))
            if disc['cardNumber']:
                discount.set('card-number', str(disc['cardNumber']))

    return (num, amount)


def addDiscountCards(purchase, discountCards, cards, purchasetype = None):
    cardsnum = 0
    for crd in cards:
        cardsnum += 1
        discountCard = ET.SubElement(discountCards, 'discountCard')
        discountCard.text = crd['number']

        if purchasetype == None:
            card = ET.SubElement(purchase, 'card')
            card.set('number', crd['number'])
            card.set('type', "INTERNAL")
    return cardsnum


def addPayments(payments, mps, valuts):
    pCash = "cash"
    pCredit = "credit"
    pBank = "bank"
    paymentsMap = {}
    for pay in mps:
        payment = ET.SubElement(payments, 'payment')

        payment.set('amount', str(pay['sumB']))
        payment.set('description', "")
        if str(pay['valCode']) in valuts['cash']:
            if pCash not in paymentsMap:
                paymentsMap[pCash] = 0
            if str(pay['opCode']) in ['70', '74']:
                payment.set('typeClass', 'CashPaymentEntity')
                paymentsMap[pCash] += pay['sumB']
            elif str(pay['opCode']) in ['72', '76']:
                payment.set('typeClass', 'CashChangePaymentEntity')
                paymentsMap[pCash] -= pay['sumB']
        elif str(pay['valCode']) in valuts['credit']:
            if pCredit not in paymentsMap:
                paymentsMap[pCredit] = 0
            payment.set('typeClass', 'ConsumerCreditPaymentEntity')
            addPaymentCreditProperties(payment)
            paymentsMap[pCredit] += pay['sumB']
        else:
            if pBank not in paymentsMap:
                paymentsMap[pBank] = 0
            payment.set('typeClass', 'BankCardPaymentEntity')
            addPaymentBankProperties(payment)
            paymentsMap[pBank] += pay['sumB']
    return paymentsMap


def addPositions(positions, invpos):
    for pos in invpos:
        position = ET.SubElement(positions, 'position')

        position.set('amount', str(pos['posSum']))
        position.set('barCode', str(pos['barCode']))
        position.set('cost', str(pos['price']))
        position.set('costWithDiscount', str(pos['posSum']))
        position.set('count', str(pos['quant']))
        position.set('dateCommit', formatDate(pos['posTime']))
        position.set('departNumber', str(pos['deptCode']))
        position.set('discountValue', str(pos['modSum'] - pos['posSum']))
        position.set('goodsCode', str(pos['inventCode']))
        position.set('nds', str(pos['vatrate1']))
        position.set('ndsSum', str(pos['vatsum1']))
        position.set('order', str(pos['posNum']))

        if pos['excisetype']:
            prop = ET.SubElement(position, 'plugin-property')
            prop.set('key', 'excise-token')
            prop.set('value', str(pos['excisemark']))

            exciseType = str(pos['excisetype']).lower()
            if exciseType == "shoes":
                exciseType = "footwear"

            prop = ET.SubElement(position, 'plugin-property')
            prop.set('key', 'marktype')
            prop.set('value', exciseType)


def commonPurchase(purchases, check, cashcode, shopcode):
    purchase = ET.SubElement(purchases, 'purchase')

    purchase.set('begintime', formatDate(check['timeBeg']))
    purchase.set('cash', str(cashcode))
    purchase.set('inn', '')
    purchase.set('number', str(check['docNum']))
    purchase.set('saletime', formatDate(check['timeEnd']))
    purchase.set('shift', str(check['shift']))
    purchase.set('shop', shopcode)

    if str(check['docType']) == '1':
        purchase.set('operationType', 'true')
    else:
        purchase.set('operationType', 'false')

    return purchase


def addPaymentPurchase(purchases, check, cashcode, shopcode, purchNum, valuts):
    purchase = commonPurchase(purchases, check, cashcode, shopcode)
    purchase.set('amount', str(check['docSum']))
    purchase.set('discountAmount', str(check['modSum'] - check['docSum']))
    purchase.set('operDay', formatDate(check['timeBeg']))

    if check['shift'] in glUserNames:
        if check['userCode'] in glUserNames[check['shift']]:
            userName = glUserNames[check['shift']][check['userCode']]
            purchase.set('userName', userName if userName else "")
        else:
            purchase.set('userName', "SHIFT_HASNT_CASHIER_WITH_CODE : %s" % check['userCode'])
    else:
        purchase.set('userName', "BAD_SHIFT : %s" % check['shift'])

    purchase.set('tabNumber', str(check['userCode']))

    positions = ET.SubElement(purchase, 'positions')
    payments = ET.SubElement(purchase, 'payments')
    discountCards = ET.SubElement(purchase, 'discountCards')

    addPositions(positions, check['inventPositions'])
    paymentsMap = addPayments(payments, check['moneyPositions'], valuts)

    if addDiscountCards(purchase, discountCards, check['cardPositions']) == 0:
        purchase.remove(discountCards)

    zrepFieldsForm(paymentsMap, check)

    purchNum += 1
    return purchNum


def addDiscountPurchase(purchases, check, cashcode, shopcode, discNum):
    amount = 0
    discNum += 1

    purchase = commonPurchase(purchases, check, cashcode, shopcode)
    purchase.set('discountValueTotal', str(check['modSum'] - check['docSum']))
    purchase.set('transactionTime', formatDate(check['timeBeg']))

    pr = 0
    for invent in check['inventPositions']:
        tmp, tmpamount = addDiscounts(purchase, invent)
        pr += tmp
        amount += tmpamount
    if not pr:
        discNum -= 1
        purchases.remove(purchase)
    else:
        zrepFieldsFormDiscount(amount, check)

    addDiscountCards(purchase, purchase, check['cardPositions'], 'disc')
    return discNum


def removeRetsInTextTags(xml):
    return re.sub(r'>\n\s*([^<>\s]*.[^<>\s]*)\n\s*</', '>\g<1></', xml)


def run(salesFunc, securityLogsFunc, shiftsFunc, cash,  node, workDirectory, currentSales):
    global glUserNames
    global saleFiles
    log = logger.getUnloadLogger("Node", node.id)
    log.info(logInfo + "START. Cash #" + str(cash.code))

    valuts = {'cash' : '1', 'credit' : '0'}
    for typeValut in valuts:
        if typeValut in node.currentConverter.paymentMapper:
            value = node.currentConverter.paymentMapper[typeValut]
        else:
            value = valuts[typeValut]
        valuts[typeValut] = [x.strip() for x in value.split(',') if x]
        log.info("type of payment - '%s', code valut - %s" % (typeValut, valuts[typeValut]))

    zreports = None

    for shift in shiftsFunc():
        zrepFillShifts(str(shift['shift']))
        if shift['timeEnd'] != None:
            if zreports == None:
                zreports = ET.Element('reports')
        shopCode = shift['shopcode']
        glUserNames[shift['shift']] = {}
        for user in shift['users']:
            glUserNames[shift['shift']][user['usercode']] = user['username']

    purchases = ET.Element('purchases')
    tree = ET.ElementTree(purchases)

    discPurchases = ET.Element('purchases')
    tree_disc = ET.ElementTree(discPurchases)

    check = None
    purchNum = 0
    discNum = 0
    for check in salesFunc():
        if check['closed'] == 1:
                if check['docType'] in [1, 2, 25]:
                    purchNum = addPaymentPurchase(purchases, check, cash.code, shopCode, purchNum, valuts)
                    discNum = addDiscountPurchase(discPurchases, check, cash.code, shopCode, discNum)
                elif check['docType'] in [3, 4]:
                    zresCashIncomeOutcome(check)

    repdate = datetime.datetime.now()
    datestr = repdate.strftime('%d-%m-%Y_%H-%M-%S')

    payPurchasesFile = '%s/purchases-%s_%s.xml' % (workDirectory, datestr, cash.code)

    if zreports != None:
        zNum = 0
        ztree = ET.ElementTree(zreports)
        zreportsFile = '%s/zreports-%s_%s.xml' % (workDirectory, datestr, cash.code)

        for shift in shiftsFunc():
            if shift['timeEnd'] != None:
                addZReport(zreports, shift, node)
                zNum += 1

        zreports.set('count', str(zNum))

        sortAttribute(ztree.iter())
    sortAttribute(tree_disc.iter())
    sortAttribute(tree.iter())
    if check:
        purchases.set('count', str(purchNum))
        discPurchases.set('count', str(discNum))

        pretty_xml = xml.dom.minidom.parseString(ET.tostring(tree.getroot(), encoding='utf-8')).toprettyxml()
        pretty_xml = removeRetsInTextTags(pretty_xml)
        with codecs.open(payPurchasesFile, "w", "utf-8") as stream:
            stream.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" + '\n')
            stream.write(re.sub(r'<\?.*\?>\n', '', pretty_xml))
            saleFiles.add(payPurchasesFile)

        if node.currentConverter.formdiscounts:
            pretty_xml = xml.dom.minidom.parseString(ET.tostring(tree_disc.getroot(), encoding="UTF-8")).toprettyxml()
            pretty_xml = removeRetsInTextTags(pretty_xml)

            discPurchasesFile = '%s/discounts-%s_%s.xml' % (workDirectory, datestr, cash.code)
            with codecs.open(discPurchasesFile, "w", "utf-8") as stream:
                stream.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" + '\n')
                stream.write(re.sub(r'<\?.*\?>\n', '', pretty_xml))
                saleFiles.add(discPurchasesFile)

        if zreports != None:
            pretty_xml = xml.dom.minidom.parseString(ET.tostring(ztree.getroot(), encoding="UTF-8")).toprettyxml()
            pretty_xml = removeRetsInTextTags(pretty_xml)
            with codecs.open(zreportsFile, "w", "utf-8") as stream:
                stream.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" + '\n')
                stream.write(re.sub(r'<\?.*\?>\n', '', pretty_xml))
                saleFiles.add(zreportsFile)

    converterSalesReadyFile = open('%s/ready' % (workDirectory), 'w')
    converterSalesReadyFile.close()
    salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
    salesReadyFile.close()
    saleFiles.add(converterSalesReadyFile.name)
    saleFiles.add(salesReadyFile.name)

    log.info(logInfo + "FINISH")

def sortAttribute(elements):
    for el in elements:
        attrib = el.attrib
        if len(attrib) > 1:
            attribs = sorted(attrib.items())
            attrib.clear()
            attrib.update(attribs)

def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, node.currentConverter.path, backupDir=backupDir,
            rewrite=node.currentConverter.rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    return False, []


def getSaleFiles():
    global saleFiles
    return saleFiles
