# -*- coding: UTF-8 -*-

from unload.node.nodeparams import (AifConverter, AifXmlConverter, ShtrihConverter, SmagConverter, Cash, AtolAstorConverter, AtolConverter, OracleConverter, CrystalConverter,
                                   NS2000Converter, PilotConverter, SapConverter, PilotOriginalConverter)

def getConverterByName(converterName, convertersConfig, salePath, flagPath, onlineSalePath, rewriteDestFile):
    converter = None

    charset = convertersConfig.getCharset(converterName)

    if converterName in ['aif', 'ARTIXINTERCHANGE']:
        dataType = convertersConfig.getString('dataType', 'sales')
        withClosedShifts = convertersConfig.getBool('withClosedShifts', True)
        converter = AifConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile,
                                 dataType=dataType, withClosedShifts=withClosedShifts, onlineSalePath=onlineSalePath)
    elif converterName in ['aif_xml', 'ARTIXINTERCHANGE_XML']:
        dataType = convertersConfig.getString('dataType', 'sales')
        withClosedShifts = convertersConfig.getBool('withClosedShifts', True)
        converter = AifXmlConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile,
                                 dataType=dataType, withClosedShifts=withClosedShifts, onlineSalePath=onlineSalePath)
    elif converterName in ['shtrih', 'SHTRIHM']:
        useActorCodeForMoneyOperations = convertersConfig.getBoolWithoutDefaultValue('useActorCodeForMoneyOperations') # В кассе по умолчанию True, КС - False
        discountsDetail = convertersConfig.getBoolWithoutDefaultValue('discountsDetail') # В кассе по умолчанию True, КС - False
        unloadShortDateFormat = convertersConfig.getBool('unloadShortDateFormat', False)
        unloadCardType = convertersConfig.getBool('unloadCardType', True)
        defaultCardType = convertersConfig.getString('defaultCardType', '')
        unloadPaymentCommission = convertersConfig.getBool('unloadPaymentCommission', False)
        delimiter = convertersConfig.getString('delimiter', ';')
        unloadPaymentByCard = convertersConfig.getBool('unloadPaymentByCard', False)
        unloadBonuses = convertersConfig.getBool('unloadBonuses', False)
        useTransaction300ForDocuments = convertersConfig.getBool('useTransaction300ForDocuments', True)
        unloadExciseMark = convertersConfig.getBool('unloadExciseMark', False)
        unloadShiftsFromKkm = convertersConfig.getBool('unloadShiftsFromKkm', False)
        checkNumTransaction = convertersConfig.getString('checkNumTransaction', '')
        unloadNotOpenedShift = convertersConfig.getBool('unloadNotOpenedShift', False)
        useCheckCashierCodeForStorno = convertersConfig.getBool('useCheckCashierCodeForStorno', False)
        unloadInventTransactionForCertificate = convertersConfig.getBool('unloadInventTransactionForCertificate', True)
        saveRegistry = convertersConfig.getBool('saveRegistry', False)
        registryFilePath = convertersConfig.getString('registryFilePath', '/tmp/sales/sales.reg')
        converter = ShtrihConverter(salePath, discountsDetail=discountsDetail, flag=flagPath, rewriteDestFile=rewriteDestFile,
                                    useActorCodeForMoneyOperations=useActorCodeForMoneyOperations, unloadShortDateFormat=unloadShortDateFormat,
                                    unloadCardType=unloadCardType, defaultCardType=defaultCardType, charset=charset, delimiter=delimiter,
                                    unloadPaymentByCard=unloadPaymentByCard, unloadBonuses=unloadBonuses, unloadPaymentCommission=unloadPaymentCommission,
                                    useTransaction300ForDocuments=useTransaction300ForDocuments, unloadExciseMark=unloadExciseMark,
                                    unloadShiftsFromKkm=unloadShiftsFromKkm, checkNumTransaction=checkNumTransaction, unloadNotOpenedShift=unloadNotOpenedShift,
                                    useCheckCashierCodeForStorno=useCheckCashierCodeForStorno,
                                    unloadInventTransactionForCertificate = unloadInventTransactionForCertificate,
                                    saveRegistry=saveRegistry, registryFilePath=registryFilePath)
    elif converterName in ['atol-astor', 'ATOL-ASTOR']:
        discountsDetail = convertersConfig.getBoolWithoutDefaultValue('discountsDetail') # В кассе по умолчанию True, КС - False
        converter = AtolAstorConverter(salePath, discountsDetail=discountsDetail, flag=flagPath, rewriteDestFile=rewriteDestFile,
                                charset=charset, onlineSalePath=onlineSalePath)
    elif converterName in ['atol', 'ATOL']:
        discountsDetail = convertersConfig.getBoolWithoutDefaultValue('discountsDetail') # В кассе по умолчанию True, КС - False
        converter = AtolConverter(salePath, discountsDetail=discountsDetail, flag=flagPath, rewriteDestFile=rewriteDestFile,
                                charset=charset, onlineSalePath=onlineSalePath)
    elif converterName in ['smag', 'SUPERMAG']:
        format = convertersConfig.getFormat('dat') # В кассе настройка fileType, КС - format
        appendSaleFiles = convertersConfig.getBool('appendSaleFiles', False)
        valutMapper = convertersConfig.getValutMapper()
        distributePaymentsForPositions = convertersConfig.getBool('distributePaymentsForPositions', False)
        converter = SmagConverter(salePath, format=format, valutMapper = valutMapper, flag=flagPath, rewriteDestFile=rewriteDestFile,
                                  onlineSalePath=onlineSalePath, charset=charset,
                                  appendSaleFiles=appendSaleFiles, distributePaymentsForPositions=distributePaymentsForPositions)
    elif converterName in ['oracle', 'ORACLE']:
        valutMapper = convertersConfig.getValutMapper()
        measuresMapper = convertersConfig.getOracleMeasureMappers()
        converter = OracleConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile, measuresMapper = measuresMapper, valutMapper = valutMapper)
    elif converterName in ['crystal', 'CRYSTAL']:
        formdiscounts = convertersConfig.getBool('formdiscounts', True)
        paymentMapper = convertersConfig.getPaymentMapper()
        converter = CrystalConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile, formdiscounts=formdiscounts, paymentMapper=paymentMapper)
    elif converterName in ['ns2000', 'NS2000']:
        converter = NS2000Converter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile, charset=charset)
    elif converterName in ['pilot', 'PILOT']:
        converter = PilotConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile, charset=charset)
    elif converterName in ['sap', 'SAP']:
        rootString = convertersConfig.getString('rootString', '')
        converter = SapConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile, onlineSalePath=onlineSalePath, rootString=rootString)
    elif converterName in ['pilot-original', 'PILOT-ORIGINAL']:
        converter = PilotOriginalConverter(salePath, flag=flagPath, rewriteDestFile=rewriteDestFile, charset=charset)
    return converter
